# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    AggregationData,
    Alert,
    AlertAlertType,
    AlertPagePublic,
    AlertPublic,
    AlertPublicAlertType,
    AlertTrigger,
    AlertTriggerConfig,
    AlertTriggerConfigPublic,
    AlertTriggerConfigPublicType,
    AlertTriggerConfigType,
    AlertTriggerConfigWrite,
    AlertTriggerConfigWriteType,
    AlertTriggerEventType,
    AlertTriggerPublic,
    AlertTriggerPublicEventType,
    AlertTriggerWrite,
    AlertTriggerWriteEventType,
    AlertWrite,
    AlertWriteAlertType,
    AnnotationQueue,
    AnnotationQueueBatch,
    AnnotationQueueItemIds,
    AnnotationQueuePagePublic,
    AnnotationQueuePublic,
    AnnotationQueuePublicScope,
    AnnotationQueueReviewer,
    AnnotationQueueReviewerPublic,
    AnnotationQueueScope,
    AnnotationQueueWrite,
    AnnotationQueueWriteScope,
    AssistantMessage,
    AssistantMessageRole,
    Attachment,
    AttachmentPage,
    AudioUrl,
    AudioUrlPublic,
    AudioUrlWrite,
    AuthDetailsHolder,
    AutomationRuleEvaluator,
    AutomationRuleEvaluatorAction,
    AutomationRuleEvaluatorLlmAsJudge,
    AutomationRuleEvaluatorLlmAsJudgePublic,
    AutomationRuleEvaluatorLlmAsJudgeWrite,
    AutomationRuleEvaluatorObjectObjectPublic,
    AutomationRuleEvaluatorObjectObjectPublicAction,
    AutomationRuleEvaluatorObjectObjectPublic_LlmAsJudge,
    AutomationRuleEvaluatorObjectObjectPublic_SpanLlmAsJudge,
    AutomationRuleEvaluatorObjectObjectPublic_SpanUserDefinedMetricPython,
    AutomationRuleEvaluatorObjectObjectPublic_TraceThreadLlmAsJudge,
    AutomationRuleEvaluatorObjectObjectPublic_TraceThreadUserDefinedMetricPython,
    AutomationRuleEvaluatorObjectObjectPublic_UserDefinedMetricPython,
    AutomationRuleEvaluatorPagePublic,
    AutomationRuleEvaluatorPublic,
    AutomationRuleEvaluatorPublicAction,
    AutomationRuleEvaluatorPublic_LlmAsJudge,
    AutomationRuleEvaluatorPublic_SpanLlmAsJudge,
    AutomationRuleEvaluatorPublic_SpanUserDefinedMetricPython,
    AutomationRuleEvaluatorPublic_TraceThreadLlmAsJudge,
    AutomationRuleEvaluatorPublic_TraceThreadUserDefinedMetricPython,
    AutomationRuleEvaluatorPublic_UserDefinedMetricPython,
    AutomationRuleEvaluatorSpanLlmAsJudge,
    AutomationRuleEvaluatorSpanLlmAsJudgePublic,
    AutomationRuleEvaluatorSpanLlmAsJudgeWrite,
    AutomationRuleEvaluatorSpanUserDefinedMetricPython,
    AutomationRuleEvaluatorSpanUserDefinedMetricPythonPublic,
    AutomationRuleEvaluatorSpanUserDefinedMetricPythonWrite,
    AutomationRuleEvaluatorTraceThreadLlmAsJudge,
    AutomationRuleEvaluatorTraceThreadLlmAsJudgePublic,
    AutomationRuleEvaluatorTraceThreadLlmAsJudgeWrite,
    AutomationRuleEvaluatorTraceThreadUserDefinedMetricPython,
    AutomationRuleEvaluatorTraceThreadUserDefinedMetricPythonPublic,
    AutomationRuleEvaluatorTraceThreadUserDefinedMetricPythonWrite,
    AutomationRuleEvaluatorUpdate,
    AutomationRuleEvaluatorUpdateAction,
    AutomationRuleEvaluatorUpdateLlmAsJudge,
    AutomationRuleEvaluatorUpdateSpanLlmAsJudge,
    AutomationRuleEvaluatorUpdateSpanUserDefinedMetricPython,
    AutomationRuleEvaluatorUpdateTraceThreadLlmAsJudge,
    AutomationRuleEvaluatorUpdateTraceThreadUserDefinedMetricPython,
    AutomationRuleEvaluatorUpdateUserDefinedMetricPython,
    AutomationRuleEvaluatorUpdate_LlmAsJudge,
    AutomationRuleEvaluatorUpdate_SpanLlmAsJudge,
    AutomationRuleEvaluatorUpdate_SpanUserDefinedMetricPython,
    AutomationRuleEvaluatorUpdate_TraceThreadLlmAsJudge,
    AutomationRuleEvaluatorUpdate_TraceThreadUserDefinedMetricPython,
    AutomationRuleEvaluatorUpdate_UserDefinedMetricPython,
    AutomationRuleEvaluatorUserDefinedMetricPython,
    AutomationRuleEvaluatorUserDefinedMetricPythonPublic,
    AutomationRuleEvaluatorUserDefinedMetricPythonWrite,
    AutomationRuleEvaluatorWrite,
    AutomationRuleEvaluatorWriteAction,
    AutomationRuleEvaluatorWrite_LlmAsJudge,
    AutomationRuleEvaluatorWrite_SpanLlmAsJudge,
    AutomationRuleEvaluatorWrite_SpanUserDefinedMetricPython,
    AutomationRuleEvaluatorWrite_TraceThreadLlmAsJudge,
    AutomationRuleEvaluatorWrite_TraceThreadUserDefinedMetricPython,
    AutomationRuleEvaluatorWrite_UserDefinedMetricPython,
    AutomationRuleEvaluator_LlmAsJudge,
    AutomationRuleEvaluator_SpanLlmAsJudge,
    AutomationRuleEvaluator_SpanUserDefinedMetricPython,
    AutomationRuleEvaluator_TraceThreadLlmAsJudge,
    AutomationRuleEvaluator_TraceThreadUserDefinedMetricPython,
    AutomationRuleEvaluator_UserDefinedMetricPython,
    AvgValueStatPublic,
    BatchDelete,
    BatchDeleteByProject,
    BiInformation,
    BiInformationResponse,
    BooleanFeedbackDefinition,
    BooleanFeedbackDefinitionCreate,
    BooleanFeedbackDefinitionPublic,
    BooleanFeedbackDefinitionUpdate,
    BooleanFeedbackDetail,
    BooleanFeedbackDetailCreate,
    BooleanFeedbackDetailPublic,
    BooleanFeedbackDetailUpdate,
    BreakdownConfigPublic,
    BreakdownConfigPublicField,
    CategoricalFeedbackDefinition,
    CategoricalFeedbackDefinitionCreate,
    CategoricalFeedbackDefinitionPublic,
    CategoricalFeedbackDefinitionUpdate,
    CategoricalFeedbackDetail,
    CategoricalFeedbackDetailCreate,
    CategoricalFeedbackDetailPublic,
    CategoricalFeedbackDetailUpdate,
    ChatCompletionChoice,
    ChatCompletionResponse,
    Check,
    CheckName,
    CheckPublic,
    CheckPublicName,
    CheckPublicResult,
    CheckResult,
    ChunkedOutputJsonNode,
    ChunkedOutputJsonNodePublic,
    ChunkedOutputJsonNodePublicType,
    ChunkedOutputJsonNodeType,
    Column,
    ColumnCompare,
    ColumnCompareTypesItem,
    ColumnPublic,
    ColumnPublicTypesItem,
    ColumnTypesItem,
    Comment,
    CommentCompare,
    CommentPublic,
    CompleteMultipartUploadRequest,
    CompleteMultipartUploadRequestEntityType,
    CompletionTokensDetails,
    CountValueStatPublic,
    DashboardPagePublic,
    DashboardPublic,
    DataPointDouble,
    DataPointNumberPublic,
    Dataset,
    DatasetExpansion,
    DatasetExpansionResponse,
    DatasetExportJobPublic,
    DatasetExportJobPublicStatus,
    DatasetItem,
    DatasetItemBatch,
    DatasetItemChangesPublic,
    DatasetItemCompare,
    DatasetItemCompareSource,
    DatasetItemFilter,
    DatasetItemFilterOperator,
    DatasetItemPageCompare,
    DatasetItemPagePublic,
    DatasetItemPublic,
    DatasetItemPublicSource,
    DatasetItemSource,
    DatasetItemUpdate,
    DatasetItemWrite,
    DatasetItemWriteSource,
    DatasetPagePublic,
    DatasetPublic,
    DatasetPublicStatus,
    DatasetPublicVisibility,
    DatasetStatus,
    DatasetVersionDiff,
    DatasetVersionDiffStats,
    DatasetVersionPagePublic,
    DatasetVersionPublic,
    DatasetVersionSummary,
    DatasetVersionSummaryPublic,
    DatasetVisibility,
    DeleteAttachmentsRequest,
    DeleteAttachmentsRequestEntityType,
    DeleteFeedbackScore,
    DeleteIdsHolder,
    Delta,
    ErrorCountWithDeviation,
    ErrorCountWithDeviationDetailed,
    ErrorInfo,
    ErrorInfoExperimentItemBulkWriteView,
    ErrorInfoPublic,
    ErrorInfoWrite,
    ErrorMessage,
    ErrorMessageDetail,
    ErrorMessageDetailed,
    ErrorMessagePublic,
    Experiment,
    ExperimentGroupAggregationsResponse,
    ExperimentGroupResponse,
    ExperimentItem,
    ExperimentItemBulkRecord,
    ExperimentItemBulkRecordExperimentItemBulkWriteView,
    ExperimentItemBulkUpload,
    ExperimentItemCompare,
    ExperimentItemCompareTraceVisibilityMode,
    ExperimentItemPublic,
    ExperimentItemPublicTraceVisibilityMode,
    ExperimentItemReference,
    ExperimentItemReferencePublic,
    ExperimentItemTraceVisibilityMode,
    ExperimentPagePublic,
    ExperimentPublic,
    ExperimentPublicStatus,
    ExperimentPublicType,
    ExperimentScore,
    ExperimentScorePublic,
    ExperimentScoreWrite,
    ExperimentStatus,
    ExperimentType,
    ExperimentUpdate,
    ExperimentUpdateStatus,
    ExperimentUpdateType,
    ExportTraceServiceRequest,
    Feedback,
    FeedbackCreate,
    FeedbackCreate_Boolean,
    FeedbackCreate_Categorical,
    FeedbackCreate_Numerical,
    FeedbackDefinitionPagePublic,
    FeedbackObjectPublic,
    FeedbackObjectPublic_Boolean,
    FeedbackObjectPublic_Categorical,
    FeedbackObjectPublic_Numerical,
    FeedbackPublic,
    FeedbackPublic_Boolean,
    FeedbackPublic_Categorical,
    FeedbackPublic_Numerical,
    FeedbackScore,
    FeedbackScoreAverage,
    FeedbackScoreAverageDetailed,
    FeedbackScoreAveragePublic,
    FeedbackScoreBatch,
    FeedbackScoreBatchItem,
    FeedbackScoreBatchItemSource,
    FeedbackScoreBatchItemThread,
    FeedbackScoreBatchItemThreadSource,
    FeedbackScoreCompare,
    FeedbackScoreCompareSource,
    FeedbackScoreExperimentItemBulkWriteView,
    FeedbackScoreExperimentItemBulkWriteViewSource,
    FeedbackScoreNames,
    FeedbackScorePublic,
    FeedbackScorePublicSource,
    FeedbackScoreSource,
    FeedbackUpdate,
    FeedbackUpdate_Boolean,
    FeedbackUpdate_Categorical,
    FeedbackUpdate_Numerical,
    Feedback_Boolean,
    Feedback_Categorical,
    Feedback_Numerical,
    Function,
    FunctionCall,
    GroupContent,
    GroupContentWithAggregations,
    GroupDetail,
    GroupDetails,
    Guardrail,
    GuardrailBatch,
    GuardrailName,
    GuardrailResult,
    GuardrailWrite,
    GuardrailWriteName,
    GuardrailWriteResult,
    GuardrailsValidation,
    GuardrailsValidationPublic,
    IdsHolder,
    ImageUrl,
    ImageUrlPublic,
    ImageUrlWrite,
    JsonListString,
    JsonListStringCompare,
    JsonListStringExperimentItemBulkWriteView,
    JsonListStringPublic,
    JsonListStringWrite,
    JsonNode,
    JsonNodeDetail,
    JsonNodePublic,
    JsonNodeWrite,
    JsonSchema,
    LlmAsJudgeCode,
    LlmAsJudgeCodePublic,
    LlmAsJudgeCodeWrite,
    LlmAsJudgeMessage,
    LlmAsJudgeMessageContent,
    LlmAsJudgeMessageContentPublic,
    LlmAsJudgeMessageContentWrite,
    LlmAsJudgeMessagePublic,
    LlmAsJudgeMessagePublicRole,
    LlmAsJudgeMessageRole,
    LlmAsJudgeMessageWrite,
    LlmAsJudgeMessageWriteRole,
    LlmAsJudgeModelParameters,
    LlmAsJudgeModelParametersPublic,
    LlmAsJudgeModelParametersWrite,
    LlmAsJudgeOutputSchema,
    LlmAsJudgeOutputSchemaPublic,
    LlmAsJudgeOutputSchemaPublicType,
    LlmAsJudgeOutputSchemaType,
    LlmAsJudgeOutputSchemaWrite,
    LlmAsJudgeOutputSchemaWriteType,
    LogItem,
    LogItemLevel,
    LogPage,
    ManualEvaluationRequest,
    ManualEvaluationRequestEntityType,
    ManualEvaluationResponse,
    Message,
    MultipartUploadPart,
    NumericalFeedbackDefinition,
    NumericalFeedbackDefinitionCreate,
    NumericalFeedbackDefinitionPublic,
    NumericalFeedbackDefinitionUpdate,
    NumericalFeedbackDetail,
    NumericalFeedbackDetailCreate,
    NumericalFeedbackDetailPublic,
    NumericalFeedbackDetailUpdate,
    OllamaConnectionTestResponse,
    OllamaInstanceBaseUrlRequest,
    OllamaModel,
    Optimization,
    OptimizationPagePublic,
    OptimizationPublic,
    OptimizationPublicStatus,
    OptimizationStatus,
    OptimizationStudioConfig,
    OptimizationStudioConfigPublic,
    OptimizationStudioConfigWrite,
    OptimizationStudioLog,
    OptimizationWrite,
    OptimizationWriteStatus,
    PageColumns,
    PercentageValueStatPublic,
    PercentageValues,
    PercentageValuesDetailed,
    PercentageValuesPublic,
    Project,
    ProjectDetailed,
    ProjectDetailedVisibility,
    ProjectMetricResponsePublic,
    ProjectMetricResponsePublicInterval,
    ProjectMetricResponsePublicMetricType,
    ProjectPagePublic,
    ProjectPublic,
    ProjectPublicVisibility,
    ProjectReference,
    ProjectReferencePublic,
    ProjectStatItemObjectPublic,
    ProjectStatItemObjectPublic_Avg,
    ProjectStatItemObjectPublic_Count,
    ProjectStatItemObjectPublic_Percentage,
    ProjectStatsPublic,
    ProjectStatsSummary,
    ProjectStatsSummaryItem,
    ProjectVisibility,
    Prompt,
    PromptDetail,
    PromptDetailTemplateStructure,
    PromptPagePublic,
    PromptPublic,
    PromptPublicTemplateStructure,
    PromptTemplateStructure,
    PromptTokensDetails,
    PromptType,
    PromptVersion,
    PromptVersionDetail,
    PromptVersionDetailTemplateStructure,
    PromptVersionDetailType,
    PromptVersionLink,
    PromptVersionLinkPublic,
    PromptVersionLinkWrite,
    PromptVersionPagePublic,
    PromptVersionPublic,
    PromptVersionPublicTemplateStructure,
    PromptVersionPublicType,
    PromptVersionTemplateStructure,
    PromptVersionType,
    PromptVersionUpdate,
    ProviderApiKey,
    ProviderApiKeyPagePublic,
    ProviderApiKeyProvider,
    ProviderApiKeyPublic,
    ProviderApiKeyPublicProvider,
    ResponseFormat,
    ResponseFormatType,
    Result,
    ResultsNumberPublic,
    ScoreName,
    ServiceTogglesConfig,
    Span,
    SpanBatch,
    SpanEnrichmentOptions,
    SpanExperimentItemBulkWriteView,
    SpanExperimentItemBulkWriteViewType,
    SpanFilter,
    SpanFilterOperator,
    SpanFilterPublic,
    SpanFilterPublicOperator,
    SpanFilterWrite,
    SpanFilterWriteOperator,
    SpanLlmAsJudgeCode,
    SpanLlmAsJudgeCodePublic,
    SpanLlmAsJudgeCodeWrite,
    SpanPagePublic,
    SpanPublic,
    SpanPublicType,
    SpanType,
    SpanUpdate,
    SpanUpdateType,
    SpanUserDefinedMetricPythonCode,
    SpanUserDefinedMetricPythonCodePublic,
    SpanUserDefinedMetricPythonCodeWrite,
    SpanWrite,
    SpanWriteType,
    SpansCountResponse,
    StartMultipartUploadResponse,
    StreamOptions,
    StudioEvaluation,
    StudioEvaluationPublic,
    StudioEvaluationWrite,
    StudioLlmModel,
    StudioLlmModelPublic,
    StudioLlmModelWrite,
    StudioMessage,
    StudioMessagePublic,
    StudioMessageWrite,
    StudioMetric,
    StudioMetricPublic,
    StudioMetricWrite,
    StudioOptimizer,
    StudioOptimizerPublic,
    StudioOptimizerWrite,
    StudioPrompt,
    StudioPromptPublic,
    StudioPromptWrite,
    TokenUsageNames,
    Tool,
    ToolCall,
    ToolCallType,
    ToolType,
    Trace,
    TraceBatch,
    TraceCountResponse,
    TraceEnrichmentOptions,
    TraceExperimentItemBulkWriteView,
    TraceFilter,
    TraceFilterOperator,
    TraceFilterPublic,
    TraceFilterPublicOperator,
    TraceFilterWrite,
    TraceFilterWriteOperator,
    TracePagePublic,
    TracePublic,
    TracePublicVisibilityMode,
    TraceThread,
    TraceThreadFilter,
    TraceThreadFilterOperator,
    TraceThreadFilterPublic,
    TraceThreadFilterPublicOperator,
    TraceThreadFilterWrite,
    TraceThreadFilterWriteOperator,
    TraceThreadIdentifier,
    TraceThreadLlmAsJudgeCode,
    TraceThreadLlmAsJudgeCodePublic,
    TraceThreadLlmAsJudgeCodeWrite,
    TraceThreadPage,
    TraceThreadStatus,
    TraceThreadUpdate,
    TraceThreadUserDefinedMetricPythonCode,
    TraceThreadUserDefinedMetricPythonCodePublic,
    TraceThreadUserDefinedMetricPythonCodeWrite,
    TraceUpdate,
    TraceVisibilityMode,
    TraceWrite,
    Usage,
    UserDefinedMetricPythonCode,
    UserDefinedMetricPythonCodePublic,
    UserDefinedMetricPythonCodeWrite,
    ValueEntry,
    ValueEntryCompare,
    ValueEntryCompareSource,
    ValueEntryExperimentItemBulkWriteView,
    ValueEntryExperimentItemBulkWriteViewSource,
    ValueEntryPublic,
    ValueEntryPublicSource,
    ValueEntrySource,
    VideoUrl,
    VideoUrlPublic,
    VideoUrlWrite,
    Webhook,
    WebhookExamples,
    WebhookPublic,
    WebhookTestResult,
    WebhookTestResultStatus,
    WebhookWrite,
    WelcomeWizardTracking,
    WorkspaceConfiguration,
    WorkspaceMetricRequest,
    WorkspaceMetricResponse,
    WorkspaceMetricsSummaryRequest,
    WorkspaceMetricsSummaryResponse,
    WorkspaceNameHolder,
    WorkspaceSpansCount,
    WorkspaceTraceCount,
)
from .errors import (
    BadGatewayError,
    BadRequestError,
    ConflictError,
    ForbiddenError,
    InternalServerError,
    NotFoundError,
    NotImplementedError,
    ServiceUnavailableError,
    UnauthorizedError,
    UnprocessableEntityError,
)
from . import (
    alerts,
    annotation_queues,
    attachments,
    automation_rule_evaluators,
    chat_completions,
    check,
    dashboards,
    datasets,
    experiments,
    feedback_definitions,
    guardrails,
    llm_provider_key,
    manual_evaluation,
    ollama,
    open_telemetry_ingestion,
    optimizations,
    projects,
    prompts,
    redirect,
    service_toggles,
    spans,
    system_usage,
    traces,
    welcome_wizard,
    workspaces,
)
from .alerts import GetWebhookExamplesRequestAlertType
from .attachments import (
    AttachmentListRequestEntityType,
    DownloadAttachmentRequestEntityType,
    StartMultipartUploadRequestEntityType,
    UploadAttachmentRequestEntityType,
)
from .client import AsyncOpikApi, OpikApi
from .datasets import DatasetUpdateVisibility, DatasetWriteVisibility
from .environment import OpikApiEnvironment
from .experiments import ExperimentWriteStatus, ExperimentWriteType
from .feedback_definitions import FindFeedbackDefinitionsRequestType
from .llm_provider_key import ProviderApiKeyWriteProvider
from .optimizations import OptimizationUpdateStatus
from .projects import (
    ProjectMetricRequestPublicInterval,
    ProjectMetricRequestPublicMetricType,
    ProjectUpdateVisibility,
    ProjectWriteVisibility,
)
from .prompts import CreatePromptVersionDetailTemplateStructure, PromptWriteTemplateStructure, PromptWriteType
from .spans import (
    FindFeedbackScoreNames1RequestType,
    GetSpanStatsRequestType,
    GetSpansByProjectRequestType,
    SpanSearchStreamRequestPublicType,
)

__all__ = [
    "AggregationData",
    "Alert",
    "AlertAlertType",
    "AlertPagePublic",
    "AlertPublic",
    "AlertPublicAlertType",
    "AlertTrigger",
    "AlertTriggerConfig",
    "AlertTriggerConfigPublic",
    "AlertTriggerConfigPublicType",
    "AlertTriggerConfigType",
    "AlertTriggerConfigWrite",
    "AlertTriggerConfigWriteType",
    "AlertTriggerEventType",
    "AlertTriggerPublic",
    "AlertTriggerPublicEventType",
    "AlertTriggerWrite",
    "AlertTriggerWriteEventType",
    "AlertWrite",
    "AlertWriteAlertType",
    "AnnotationQueue",
    "AnnotationQueueBatch",
    "AnnotationQueueItemIds",
    "AnnotationQueuePagePublic",
    "AnnotationQueuePublic",
    "AnnotationQueuePublicScope",
    "AnnotationQueueReviewer",
    "AnnotationQueueReviewerPublic",
    "AnnotationQueueScope",
    "AnnotationQueueWrite",
    "AnnotationQueueWriteScope",
    "AssistantMessage",
    "AssistantMessageRole",
    "AsyncOpikApi",
    "Attachment",
    "AttachmentListRequestEntityType",
    "AttachmentPage",
    "AudioUrl",
    "AudioUrlPublic",
    "AudioUrlWrite",
    "AuthDetailsHolder",
    "AutomationRuleEvaluator",
    "AutomationRuleEvaluatorAction",
    "AutomationRuleEvaluatorLlmAsJudge",
    "AutomationRuleEvaluatorLlmAsJudgePublic",
    "AutomationRuleEvaluatorLlmAsJudgeWrite",
    "AutomationRuleEvaluatorObjectObjectPublic",
    "AutomationRuleEvaluatorObjectObjectPublicAction",
    "AutomationRuleEvaluatorObjectObjectPublic_LlmAsJudge",
    "AutomationRuleEvaluatorObjectObjectPublic_SpanLlmAsJudge",
    "AutomationRuleEvaluatorObjectObjectPublic_SpanUserDefinedMetricPython",
    "AutomationRuleEvaluatorObjectObjectPublic_TraceThreadLlmAsJudge",
    "AutomationRuleEvaluatorObjectObjectPublic_TraceThreadUserDefinedMetricPython",
    "AutomationRuleEvaluatorObjectObjectPublic_UserDefinedMetricPython",
    "AutomationRuleEvaluatorPagePublic",
    "AutomationRuleEvaluatorPublic",
    "AutomationRuleEvaluatorPublicAction",
    "AutomationRuleEvaluatorPublic_LlmAsJudge",
    "AutomationRuleEvaluatorPublic_SpanLlmAsJudge",
    "AutomationRuleEvaluatorPublic_SpanUserDefinedMetricPython",
    "AutomationRuleEvaluatorPublic_TraceThreadLlmAsJudge",
    "AutomationRuleEvaluatorPublic_TraceThreadUserDefinedMetricPython",
    "AutomationRuleEvaluatorPublic_UserDefinedMetricPython",
    "AutomationRuleEvaluatorSpanLlmAsJudge",
    "AutomationRuleEvaluatorSpanLlmAsJudgePublic",
    "AutomationRuleEvaluatorSpanLlmAsJudgeWrite",
    "AutomationRuleEvaluatorSpanUserDefinedMetricPython",
    "AutomationRuleEvaluatorSpanUserDefinedMetricPythonPublic",
    "AutomationRuleEvaluatorSpanUserDefinedMetricPythonWrite",
    "AutomationRuleEvaluatorTraceThreadLlmAsJudge",
    "AutomationRuleEvaluatorTraceThreadLlmAsJudgePublic",
    "AutomationRuleEvaluatorTraceThreadLlmAsJudgeWrite",
    "AutomationRuleEvaluatorTraceThreadUserDefinedMetricPython",
    "AutomationRuleEvaluatorTraceThreadUserDefinedMetricPythonPublic",
    "AutomationRuleEvaluatorTraceThreadUserDefinedMetricPythonWrite",
    "AutomationRuleEvaluatorUpdate",
    "AutomationRuleEvaluatorUpdateAction",
    "AutomationRuleEvaluatorUpdateLlmAsJudge",
    "AutomationRuleEvaluatorUpdateSpanLlmAsJudge",
    "AutomationRuleEvaluatorUpdateSpanUserDefinedMetricPython",
    "AutomationRuleEvaluatorUpdateTraceThreadLlmAsJudge",
    "AutomationRuleEvaluatorUpdateTraceThreadUserDefinedMetricPython",
    "AutomationRuleEvaluatorUpdateUserDefinedMetricPython",
    "AutomationRuleEvaluatorUpdate_LlmAsJudge",
    "AutomationRuleEvaluatorUpdate_SpanLlmAsJudge",
    "AutomationRuleEvaluatorUpdate_SpanUserDefinedMetricPython",
    "AutomationRuleEvaluatorUpdate_TraceThreadLlmAsJudge",
    "AutomationRuleEvaluatorUpdate_TraceThreadUserDefinedMetricPython",
    "AutomationRuleEvaluatorUpdate_UserDefinedMetricPython",
    "AutomationRuleEvaluatorUserDefinedMetricPython",
    "AutomationRuleEvaluatorUserDefinedMetricPythonPublic",
    "AutomationRuleEvaluatorUserDefinedMetricPythonWrite",
    "AutomationRuleEvaluatorWrite",
    "AutomationRuleEvaluatorWriteAction",
    "AutomationRuleEvaluatorWrite_LlmAsJudge",
    "AutomationRuleEvaluatorWrite_SpanLlmAsJudge",
    "AutomationRuleEvaluatorWrite_SpanUserDefinedMetricPython",
    "AutomationRuleEvaluatorWrite_TraceThreadLlmAsJudge",
    "AutomationRuleEvaluatorWrite_TraceThreadUserDefinedMetricPython",
    "AutomationRuleEvaluatorWrite_UserDefinedMetricPython",
    "AutomationRuleEvaluator_LlmAsJudge",
    "AutomationRuleEvaluator_SpanLlmAsJudge",
    "AutomationRuleEvaluator_SpanUserDefinedMetricPython",
    "AutomationRuleEvaluator_TraceThreadLlmAsJudge",
    "AutomationRuleEvaluator_TraceThreadUserDefinedMetricPython",
    "AutomationRuleEvaluator_UserDefinedMetricPython",
    "AvgValueStatPublic",
    "BadGatewayError",
    "BadRequestError",
    "BatchDelete",
    "BatchDeleteByProject",
    "BiInformation",
    "BiInformationResponse",
    "BooleanFeedbackDefinition",
    "BooleanFeedbackDefinitionCreate",
    "BooleanFeedbackDefinitionPublic",
    "BooleanFeedbackDefinitionUpdate",
    "BooleanFeedbackDetail",
    "BooleanFeedbackDetailCreate",
    "BooleanFeedbackDetailPublic",
    "BooleanFeedbackDetailUpdate",
    "BreakdownConfigPublic",
    "BreakdownConfigPublicField",
    "CategoricalFeedbackDefinition",
    "CategoricalFeedbackDefinitionCreate",
    "CategoricalFeedbackDefinitionPublic",
    "CategoricalFeedbackDefinitionUpdate",
    "CategoricalFeedbackDetail",
    "CategoricalFeedbackDetailCreate",
    "CategoricalFeedbackDetailPublic",
    "CategoricalFeedbackDetailUpdate",
    "ChatCompletionChoice",
    "ChatCompletionResponse",
    "Check",
    "CheckName",
    "CheckPublic",
    "CheckPublicName",
    "CheckPublicResult",
    "CheckResult",
    "ChunkedOutputJsonNode",
    "ChunkedOutputJsonNodePublic",
    "ChunkedOutputJsonNodePublicType",
    "ChunkedOutputJsonNodeType",
    "Column",
    "ColumnCompare",
    "ColumnCompareTypesItem",
    "ColumnPublic",
    "ColumnPublicTypesItem",
    "ColumnTypesItem",
    "Comment",
    "CommentCompare",
    "CommentPublic",
    "CompleteMultipartUploadRequest",
    "CompleteMultipartUploadRequestEntityType",
    "CompletionTokensDetails",
    "ConflictError",
    "CountValueStatPublic",
    "CreatePromptVersionDetailTemplateStructure",
    "DashboardPagePublic",
    "DashboardPublic",
    "DataPointDouble",
    "DataPointNumberPublic",
    "Dataset",
    "DatasetExpansion",
    "DatasetExpansionResponse",
    "DatasetExportJobPublic",
    "DatasetExportJobPublicStatus",
    "DatasetItem",
    "DatasetItemBatch",
    "DatasetItemChangesPublic",
    "DatasetItemCompare",
    "DatasetItemCompareSource",
    "DatasetItemFilter",
    "DatasetItemFilterOperator",
    "DatasetItemPageCompare",
    "DatasetItemPagePublic",
    "DatasetItemPublic",
    "DatasetItemPublicSource",
    "DatasetItemSource",
    "DatasetItemUpdate",
    "DatasetItemWrite",
    "DatasetItemWriteSource",
    "DatasetPagePublic",
    "DatasetPublic",
    "DatasetPublicStatus",
    "DatasetPublicVisibility",
    "DatasetStatus",
    "DatasetUpdateVisibility",
    "DatasetVersionDiff",
    "DatasetVersionDiffStats",
    "DatasetVersionPagePublic",
    "DatasetVersionPublic",
    "DatasetVersionSummary",
    "DatasetVersionSummaryPublic",
    "DatasetVisibility",
    "DatasetWriteVisibility",
    "DeleteAttachmentsRequest",
    "DeleteAttachmentsRequestEntityType",
    "DeleteFeedbackScore",
    "DeleteIdsHolder",
    "Delta",
    "DownloadAttachmentRequestEntityType",
    "ErrorCountWithDeviation",
    "ErrorCountWithDeviationDetailed",
    "ErrorInfo",
    "ErrorInfoExperimentItemBulkWriteView",
    "ErrorInfoPublic",
    "ErrorInfoWrite",
    "ErrorMessage",
    "ErrorMessageDetail",
    "ErrorMessageDetailed",
    "ErrorMessagePublic",
    "Experiment",
    "ExperimentGroupAggregationsResponse",
    "ExperimentGroupResponse",
    "ExperimentItem",
    "ExperimentItemBulkRecord",
    "ExperimentItemBulkRecordExperimentItemBulkWriteView",
    "ExperimentItemBulkUpload",
    "ExperimentItemCompare",
    "ExperimentItemCompareTraceVisibilityMode",
    "ExperimentItemPublic",
    "ExperimentItemPublicTraceVisibilityMode",
    "ExperimentItemReference",
    "ExperimentItemReferencePublic",
    "ExperimentItemTraceVisibilityMode",
    "ExperimentPagePublic",
    "ExperimentPublic",
    "ExperimentPublicStatus",
    "ExperimentPublicType",
    "ExperimentScore",
    "ExperimentScorePublic",
    "ExperimentScoreWrite",
    "ExperimentStatus",
    "ExperimentType",
    "ExperimentUpdate",
    "ExperimentUpdateStatus",
    "ExperimentUpdateType",
    "ExperimentWriteStatus",
    "ExperimentWriteType",
    "ExportTraceServiceRequest",
    "Feedback",
    "FeedbackCreate",
    "FeedbackCreate_Boolean",
    "FeedbackCreate_Categorical",
    "FeedbackCreate_Numerical",
    "FeedbackDefinitionPagePublic",
    "FeedbackObjectPublic",
    "FeedbackObjectPublic_Boolean",
    "FeedbackObjectPublic_Categorical",
    "FeedbackObjectPublic_Numerical",
    "FeedbackPublic",
    "FeedbackPublic_Boolean",
    "FeedbackPublic_Categorical",
    "FeedbackPublic_Numerical",
    "FeedbackScore",
    "FeedbackScoreAverage",
    "FeedbackScoreAverageDetailed",
    "FeedbackScoreAveragePublic",
    "FeedbackScoreBatch",
    "FeedbackScoreBatchItem",
    "FeedbackScoreBatchItemSource",
    "FeedbackScoreBatchItemThread",
    "FeedbackScoreBatchItemThreadSource",
    "FeedbackScoreCompare",
    "FeedbackScoreCompareSource",
    "FeedbackScoreExperimentItemBulkWriteView",
    "FeedbackScoreExperimentItemBulkWriteViewSource",
    "FeedbackScoreNames",
    "FeedbackScorePublic",
    "FeedbackScorePublicSource",
    "FeedbackScoreSource",
    "FeedbackUpdate",
    "FeedbackUpdate_Boolean",
    "FeedbackUpdate_Categorical",
    "FeedbackUpdate_Numerical",
    "Feedback_Boolean",
    "Feedback_Categorical",
    "Feedback_Numerical",
    "FindFeedbackDefinitionsRequestType",
    "FindFeedbackScoreNames1RequestType",
    "ForbiddenError",
    "Function",
    "FunctionCall",
    "GetSpanStatsRequestType",
    "GetSpansByProjectRequestType",
    "GetWebhookExamplesRequestAlertType",
    "GroupContent",
    "GroupContentWithAggregations",
    "GroupDetail",
    "GroupDetails",
    "Guardrail",
    "GuardrailBatch",
    "GuardrailName",
    "GuardrailResult",
    "GuardrailWrite",
    "GuardrailWriteName",
    "GuardrailWriteResult",
    "GuardrailsValidation",
    "GuardrailsValidationPublic",
    "IdsHolder",
    "ImageUrl",
    "ImageUrlPublic",
    "ImageUrlWrite",
    "InternalServerError",
    "JsonListString",
    "JsonListStringCompare",
    "JsonListStringExperimentItemBulkWriteView",
    "JsonListStringPublic",
    "JsonListStringWrite",
    "JsonNode",
    "JsonNodeDetail",
    "JsonNodePublic",
    "JsonNodeWrite",
    "JsonSchema",
    "LlmAsJudgeCode",
    "LlmAsJudgeCodePublic",
    "LlmAsJudgeCodeWrite",
    "LlmAsJudgeMessage",
    "LlmAsJudgeMessageContent",
    "LlmAsJudgeMessageContentPublic",
    "LlmAsJudgeMessageContentWrite",
    "LlmAsJudgeMessagePublic",
    "LlmAsJudgeMessagePublicRole",
    "LlmAsJudgeMessageRole",
    "LlmAsJudgeMessageWrite",
    "LlmAsJudgeMessageWriteRole",
    "LlmAsJudgeModelParameters",
    "LlmAsJudgeModelParametersPublic",
    "LlmAsJudgeModelParametersWrite",
    "LlmAsJudgeOutputSchema",
    "LlmAsJudgeOutputSchemaPublic",
    "LlmAsJudgeOutputSchemaPublicType",
    "LlmAsJudgeOutputSchemaType",
    "LlmAsJudgeOutputSchemaWrite",
    "LlmAsJudgeOutputSchemaWriteType",
    "LogItem",
    "LogItemLevel",
    "LogPage",
    "ManualEvaluationRequest",
    "ManualEvaluationRequestEntityType",
    "ManualEvaluationResponse",
    "Message",
    "MultipartUploadPart",
    "NotFoundError",
    "NotImplementedError",
    "NumericalFeedbackDefinition",
    "NumericalFeedbackDefinitionCreate",
    "NumericalFeedbackDefinitionPublic",
    "NumericalFeedbackDefinitionUpdate",
    "NumericalFeedbackDetail",
    "NumericalFeedbackDetailCreate",
    "NumericalFeedbackDetailPublic",
    "NumericalFeedbackDetailUpdate",
    "OllamaConnectionTestResponse",
    "OllamaInstanceBaseUrlRequest",
    "OllamaModel",
    "OpikApi",
    "OpikApiEnvironment",
    "Optimization",
    "OptimizationPagePublic",
    "OptimizationPublic",
    "OptimizationPublicStatus",
    "OptimizationStatus",
    "OptimizationStudioConfig",
    "OptimizationStudioConfigPublic",
    "OptimizationStudioConfigWrite",
    "OptimizationStudioLog",
    "OptimizationUpdateStatus",
    "OptimizationWrite",
    "OptimizationWriteStatus",
    "PageColumns",
    "PercentageValueStatPublic",
    "PercentageValues",
    "PercentageValuesDetailed",
    "PercentageValuesPublic",
    "Project",
    "ProjectDetailed",
    "ProjectDetailedVisibility",
    "ProjectMetricRequestPublicInterval",
    "ProjectMetricRequestPublicMetricType",
    "ProjectMetricResponsePublic",
    "ProjectMetricResponsePublicInterval",
    "ProjectMetricResponsePublicMetricType",
    "ProjectPagePublic",
    "ProjectPublic",
    "ProjectPublicVisibility",
    "ProjectReference",
    "ProjectReferencePublic",
    "ProjectStatItemObjectPublic",
    "ProjectStatItemObjectPublic_Avg",
    "ProjectStatItemObjectPublic_Count",
    "ProjectStatItemObjectPublic_Percentage",
    "ProjectStatsPublic",
    "ProjectStatsSummary",
    "ProjectStatsSummaryItem",
    "ProjectUpdateVisibility",
    "ProjectVisibility",
    "ProjectWriteVisibility",
    "Prompt",
    "PromptDetail",
    "PromptDetailTemplateStructure",
    "PromptPagePublic",
    "PromptPublic",
    "PromptPublicTemplateStructure",
    "PromptTemplateStructure",
    "PromptTokensDetails",
    "PromptType",
    "PromptVersion",
    "PromptVersionDetail",
    "PromptVersionDetailTemplateStructure",
    "PromptVersionDetailType",
    "PromptVersionLink",
    "PromptVersionLinkPublic",
    "PromptVersionLinkWrite",
    "PromptVersionPagePublic",
    "PromptVersionPublic",
    "PromptVersionPublicTemplateStructure",
    "PromptVersionPublicType",
    "PromptVersionTemplateStructure",
    "PromptVersionType",
    "PromptVersionUpdate",
    "PromptWriteTemplateStructure",
    "PromptWriteType",
    "ProviderApiKey",
    "ProviderApiKeyPagePublic",
    "ProviderApiKeyProvider",
    "ProviderApiKeyPublic",
    "ProviderApiKeyPublicProvider",
    "ProviderApiKeyWriteProvider",
    "ResponseFormat",
    "ResponseFormatType",
    "Result",
    "ResultsNumberPublic",
    "ScoreName",
    "ServiceTogglesConfig",
    "ServiceUnavailableError",
    "Span",
    "SpanBatch",
    "SpanEnrichmentOptions",
    "SpanExperimentItemBulkWriteView",
    "SpanExperimentItemBulkWriteViewType",
    "SpanFilter",
    "SpanFilterOperator",
    "SpanFilterPublic",
    "SpanFilterPublicOperator",
    "SpanFilterWrite",
    "SpanFilterWriteOperator",
    "SpanLlmAsJudgeCode",
    "SpanLlmAsJudgeCodePublic",
    "SpanLlmAsJudgeCodeWrite",
    "SpanPagePublic",
    "SpanPublic",
    "SpanPublicType",
    "SpanSearchStreamRequestPublicType",
    "SpanType",
    "SpanUpdate",
    "SpanUpdateType",
    "SpanUserDefinedMetricPythonCode",
    "SpanUserDefinedMetricPythonCodePublic",
    "SpanUserDefinedMetricPythonCodeWrite",
    "SpanWrite",
    "SpanWriteType",
    "SpansCountResponse",
    "StartMultipartUploadRequestEntityType",
    "StartMultipartUploadResponse",
    "StreamOptions",
    "StudioEvaluation",
    "StudioEvaluationPublic",
    "StudioEvaluationWrite",
    "StudioLlmModel",
    "StudioLlmModelPublic",
    "StudioLlmModelWrite",
    "StudioMessage",
    "StudioMessagePublic",
    "StudioMessageWrite",
    "StudioMetric",
    "StudioMetricPublic",
    "StudioMetricWrite",
    "StudioOptimizer",
    "StudioOptimizerPublic",
    "StudioOptimizerWrite",
    "StudioPrompt",
    "StudioPromptPublic",
    "StudioPromptWrite",
    "TokenUsageNames",
    "Tool",
    "ToolCall",
    "ToolCallType",
    "ToolType",
    "Trace",
    "TraceBatch",
    "TraceCountResponse",
    "TraceEnrichmentOptions",
    "TraceExperimentItemBulkWriteView",
    "TraceFilter",
    "TraceFilterOperator",
    "TraceFilterPublic",
    "TraceFilterPublicOperator",
    "TraceFilterWrite",
    "TraceFilterWriteOperator",
    "TracePagePublic",
    "TracePublic",
    "TracePublicVisibilityMode",
    "TraceThread",
    "TraceThreadFilter",
    "TraceThreadFilterOperator",
    "TraceThreadFilterPublic",
    "TraceThreadFilterPublicOperator",
    "TraceThreadFilterWrite",
    "TraceThreadFilterWriteOperator",
    "TraceThreadIdentifier",
    "TraceThreadLlmAsJudgeCode",
    "TraceThreadLlmAsJudgeCodePublic",
    "TraceThreadLlmAsJudgeCodeWrite",
    "TraceThreadPage",
    "TraceThreadStatus",
    "TraceThreadUpdate",
    "TraceThreadUserDefinedMetricPythonCode",
    "TraceThreadUserDefinedMetricPythonCodePublic",
    "TraceThreadUserDefinedMetricPythonCodeWrite",
    "TraceUpdate",
    "TraceVisibilityMode",
    "TraceWrite",
    "UnauthorizedError",
    "UnprocessableEntityError",
    "UploadAttachmentRequestEntityType",
    "Usage",
    "UserDefinedMetricPythonCode",
    "UserDefinedMetricPythonCodePublic",
    "UserDefinedMetricPythonCodeWrite",
    "ValueEntry",
    "ValueEntryCompare",
    "ValueEntryCompareSource",
    "ValueEntryExperimentItemBulkWriteView",
    "ValueEntryExperimentItemBulkWriteViewSource",
    "ValueEntryPublic",
    "ValueEntryPublicSource",
    "ValueEntrySource",
    "VideoUrl",
    "VideoUrlPublic",
    "VideoUrlWrite",
    "Webhook",
    "WebhookExamples",
    "WebhookPublic",
    "WebhookTestResult",
    "WebhookTestResultStatus",
    "WebhookWrite",
    "WelcomeWizardTracking",
    "WorkspaceConfiguration",
    "WorkspaceMetricRequest",
    "WorkspaceMetricResponse",
    "WorkspaceMetricsSummaryRequest",
    "WorkspaceMetricsSummaryResponse",
    "WorkspaceNameHolder",
    "WorkspaceSpansCount",
    "WorkspaceTraceCount",
    "alerts",
    "annotation_queues",
    "attachments",
    "automation_rule_evaluators",
    "chat_completions",
    "check",
    "dashboards",
    "datasets",
    "experiments",
    "feedback_definitions",
    "guardrails",
    "llm_provider_key",
    "manual_evaluation",
    "ollama",
    "open_telemetry_ingestion",
    "optimizations",
    "projects",
    "prompts",
    "redirect",
    "service_toggles",
    "spans",
    "system_usage",
    "traces",
    "welcome_wizard",
    "workspaces",
]
