# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .experiment_score import ExperimentScore
from .experiment_update_status import ExperimentUpdateStatus
from .experiment_update_type import ExperimentUpdateType
from .json_node import JsonNode


class ExperimentUpdate(UniversalBaseModel):
    name: typing.Optional[str] = None
    metadata: typing.Optional[JsonNode] = None
    tags: typing.Optional[typing.List[str]] = None
    type: typing.Optional[ExperimentUpdateType] = None
    status: typing.Optional[ExperimentUpdateStatus] = pydantic.Field(default=None)
    """
    The status of the experiment
    """

    experiment_scores: typing.Optional[typing.List[ExperimentScore]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
