# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class OllamaInstanceBaseUrlRequest(UniversalBaseModel):
    """
    Request with Ollama instance base URL for connection testing or model discovery.
    """

    base_url: str = pydantic.Field()
    """
    Base URL of the Ollama instance. May include /v1 suffix which will be automatically removed for connection testing. For inference, use the URL with /v1 suffix for OpenAI-compatible endpoints.
    """

    api_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional API key for authenticated Ollama instances. If provided, will be sent as Bearer token in Authorization header.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
