# This file was auto-generated by Fern from our API Definition.

import typing

BreakdownConfigPublicField = typing.Union[
    typing.Literal["none", "tags", "metadata", "name", "error_info", "model", "provider", "type"], typing.Any
]
