# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .assistant_message import AssistantMessage
from .delta import Delta


class ChatCompletionChoice(UniversalBaseModel):
    index: typing.Optional[int] = None
    message: typing.Optional[AssistantMessage] = None
    delta: typing.Optional[Delta] = None
    finish_reason: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
