# This file was auto-generated by Fern from our API Definition.

import typing

DatasetExportJobPublicStatus = typing.Union[typing.Literal["PENDING", "PROCESSING", "COMPLETED", "FAILED"], typing.Any]
