# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .function_call import FunctionCall
from .tool_call_type import ToolCallType


class ToolCall(UniversalBaseModel):
    id: typing.Optional[str] = None
    index: typing.Optional[int] = None
    type: typing.Optional[ToolCallType] = None
    function: typing.Optional[FunctionCall] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
